<?php declare(strict_types=1);

namespace Addons\Integrations\Pusher\Requests;

use App\Http\Requests\FormRequest;
use Illuminate\Validation\Rules\In;

use function trans;

class SettingsRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string,array>
     */
    public function rules(): array
    {
        return [
            'app_id'                    => ['integer', 'required_if:options.hosting,0'],
            'key'                       => ['required_if:options.hosting,0'],
            'secret'                    => ['required_if:options.hosting,0'],
            'options.hosting'           => ['required', new In([0, 1])],
            'options.cluster'           => ['required_if:options.hosting,0'],
            'options.host'              => ['string', 'required_if:options.hosting,1'],
            'options.port'              => ['integer', 'required_if:options.hosting,1'],
            'options.external_host'     => ['string'],
            'options.external_port'     => ['integer'],
            'options.encryption'        => ['boolean', 'required_if:options.hosting,1'],
        ];
    }

    /**
     * Get custom attributes for validator errors.
     *
     * @return array<string,string>
     */
    public function attributes(): array
    {
        return [
            'app_id'                => trans('integrations.app_id'),
            'key'                   => trans('integrations.pusher_key'),
            'secret'                => trans('integrations.pusher_secret'),
            'options.hosting'       => trans('integrations.pusher_hosting'),
            'options.cluster'       => trans('integrations.pusher_cluster'),
            'options.host'          => trans('integrations.pusher_internal_host'),
            'options.port'          => trans('ticket.port'),
            'options.external_host' => trans('integrations.pusher_external_host'),
            'options.external_port' => trans('ticket.port'),
            'options.encryption'    => trans('ticket.encryption'),
        ];
    }

    /**
     * Get custom messages for validator errors.
     *
     * @return array<string,string>
     */
    public function messages(): array
    {
        return [
            'app_id.required_if'             => trans('validation.required'),
            'key.required_if'                => trans('validation.required'),
            'secret.required_if'             => trans('validation.required'),
            'options.cluster.required_if'    => trans('validation.required'),
            'options.host.required_if'       => trans('validation.required'),
            'options.port.required_if'       => trans('validation.required'),
            'options.encryption.required_if' => trans('validation.required'),
        ];
    }
}
