<?php declare(strict_types=1);

namespace Addons\Integrations\Twitter\Requests;

use App\Http\Requests\FormRequest;
use Illuminate\Support\Facades\Lang;

class SettingsRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize(): bool
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules(): array
    {
        $rules = [];

        foreach (brand_config(null) as $brand) {
            $id = $brand->id;

            $rules['brand-settings.' . $id . '.client_id'] = [
                'nullable',
                'required_with:brand-settings.' . $id . '.client_secret'
            ];
            $rules['brand-settings.' . $id . '.client_secret'] = [
                'nullable',
                'required_with:brand-settings.' . $id . '.client_id'
            ];
        }

        return $rules;
    }

    /**
     * Get custom attributes for validator errors.
     *
     * @return array
     */
    public function attributes(): array
    {
        $attributes = [];

        foreach (brand_config(null) as $brand) {
            $id = $brand->id;

            $attributes['brand-settings.' . $id . '.client_id'] = Lang::get('integrations.api_key');
            $attributes['brand-settings.' . $id . '.client_secret'] = Lang::get('integrations.twitter_api_key_secret');
        }

        return $attributes;
    }
}
