<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports\Articles\Responses;

use Addons\Reports\Feedback\Reports\Cards\Responses\FieldTable;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Selfservice\Models\Article;
use App\Modules\Selfservice\Models\FeedbackFormFieldValue;
use App\Modules\Selfservice\Models\FeedbackLog;
use SupportPal\Purifier\Facades\Purifier;

use function e;
use function route;
use function trans;

/**
 * @extends FieldTable<Article, FeedbackLog, FeedbackFormFieldValue>
 */
class TextareaFieldTable extends FieldTable
{
    public function model()
    {
        return new Article;
    }

    public function columns(): array
    {
        return [$this->valueColumn()];
    }

    protected function valueColumn(): Column
    {
        return (new Column(trans('report.feedback_responses')))
            ->setCsvValue(fn (Article $model) => $model->feedback?->values->firstOrFail()->value ?? '')
            ->setValue(function (Article $model) {
                $text = Purifier::clean($model->feedback?->values->firstOrFail()->value ?? '');
                $route = e(route('selfservice.operator.article.edit', [$model->id]));
                $feedbackRoute = e(route('selfservice.operator.feedback.show', [$model->feedback?->id]));
                $date = e(formatDate((int) $model->feedback?->updated_at));
                $title = e($model->title);
                $ratingColour = e($model->feedback?->rating?->textColour() ?? '');
                $ratingName = e($model->feedback?->rating?->toString() ?? '');
                $LANG = [
                    'ticket.rating' => e(trans('ticket.rating')),
                    'ticket.view_feedback' => e(trans('ticket.view_feedback')),
                ];

                return <<<EOT
${text}
<div class="sp-description sp-mt-3">
    <i class="fas fa-clock"></i>&nbsp;${date} &nbsp;
    <i class="fas fa-newspaper"></i>&nbsp; <a target="_blank" href="${route}">${title}</a> &nbsp;
    - ${LANG['ticket.rating']}: <span class="${ratingColour}">${ratingName}</span>
    - <a target="_blank" href="${feedbackRoute}">${LANG['ticket.view_feedback']}</a>
</div>
EOT;
            }, false);
    }
}
