<?php declare(strict_types=1);

namespace Addons\Reports\Feedback\Reports;

use Addons\Reports\Feedback\Reports\Tickets\Responses\BooleanFieldChart;
use Addons\Reports\Feedback\Reports\Tickets\Responses\OptionsFieldChart;
use Addons\Reports\Feedback\Reports\Tickets\Responses\RatingFieldChart;
use Addons\Reports\Feedback\Reports\Tickets\Responses\TextareaFieldTable;
use Addons\Reports\Feedback\Reports\Tickets\Responses\TextFieldTable;
use App\Modules\Core\Models\CustomField;
use App\Modules\Report\Addon\Dashboards\CardCollection;
use App\Modules\Report\Addon\Dashboards\Dashboard;
use App\Modules\Report\Addon\Dashboards\Filtering\Filter;
use App\Modules\Report\Addon\Dashboards\Filtering\Timeframe;
use App\Modules\Ticket\Models\FeedbackFormField;
use UnhandledMatchError;

use function trans;

class TicketsResponses extends Dashboard
{
    public Timeframe $timeframe = Timeframe::ENABLE;
    public Filter $filter = Filter::TICKET;

    public function name(): string
    {
        return trans('report.feedback_ticket_responses');
    }

    public function description(): string
    {
        return trans('report.feedback_responses_desc');
    }

    public function loadCards(CardCollection $collection): void
    {
        $fields = FeedbackFormField::get();

        foreach ($fields as $field) {
            try {
                $card = match ($field->type) {
                    CustomField::BOOLEAN, CustomField::CHECKBOX => new BooleanFieldChart($field),
                    CustomField::CHECKLIST, CustomField::MULTIPLE, CustomField::OPTIONS, CustomField::RADIO => new OptionsFieldChart($field),
                    CustomField::DATE, CustomField::TEXT => new TextFieldTable($field),
                    CustomField::RATING => new RatingFieldChart($field),
                    CustomField::TEXTAREA => new TextareaFieldTable($field),
                };

                $card->setUriKey($card->uriKey() . '_' . $field->id);
                $collection->add($card);
            } catch (UnhandledMatchError) {
                continue;
            }
        }

        $collection->sortByBestLayout();
    }
}
