<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\FirstResponseTime;

use App\Modules\Report\Addon\Concerns\UnixTimestamp;
use App\Modules\Report\Addon\Dashboards\Cards\HtmlCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Report\Metrics\Metric;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Contracts\View\View;
use TemplateView;

use function trans;

class AverageCard extends HtmlCard
{
    use UnixTimestamp;

    public function name(): string
    {
        return trans('report.first_response_time_average');
    }

    public function html(): View
    {
        $time = (int) $this->baseQuery()->avg('report_metric_value.value');
        if ($time === 0) {
            $time = trans('report.no_data_found');
        } else {
            $time = $this->formatTime($time, false);
        }

        return TemplateView::make('Reports#TicketPerformance::time')
            ->with('time', $time);
    }

    /**
     * @return QueryBuilder<Ticket>
     */
    private function baseQuery(): QueryBuilder
    {
        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket.created_at'))
            ->join('report_metric_value', 'report_metric_value.model_id', '=', 'ticket.id')
            ->join('report_metric', 'report_metric_value.metric_id', '=', 'report_metric.id')
            ->where('report_metric.name', '=', Metric::FIRST_RESPONSE_TIME->value);
    }
}
