<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\FirstResponseTime;

use App\Http\Requests\Request;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\Cards\UnderlyingData;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketCreatedAtColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketDepartmentColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketNumberColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketSubjectColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\UserFormattedNameColumn;
use App\Modules\Ticket\Models\Ticket;
use Carbon\CarbonInterval;

use function e;
use function sprintf;
use function trans;

/**
 * @implements UnderlyingData<Ticket>
 */
abstract class BaseChart extends ChartCard implements UnderlyingData
{
    protected int $width = 6;

    public function underlyingDataTableColumns(): array
    {
        $timeColumn = (new Column(trans('report.response_time')))
            ->setId('first_response_time')
            ->setDefinition(['width' => '160px'])
            ->setCsvValue(fn($ticket) => CarbonInterval::seconds($ticket->first_response_time)->cascade()->forHumans())
            ->setValue(function ($ticket) {
                $carbon = CarbonInterval::seconds($ticket->first_response_time)->cascade();
                $fullTime = $carbon->forHumans();
                $formattedTime = $carbon->forHumans(null, true, 3);

                return sprintf('<time title="%s">%s</time>', e($fullTime), e($formattedTime));
            }, false);

        return [
            $timeColumn,
            new TicketCreatedAtColumn,
            new TicketNumberColumn,
            new TicketSubjectColumn,
            new UserFormattedNameColumn,
            new TicketDepartmentColumn,
        ];
    }

    protected function isDrillIntoChartRequest(Request $request): bool
    {
        return $request->has('label');
    }
}
