<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance;

use App\Http\Requests\Request;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Ticket;
use App\Modules\Ticket\Models\TicketDueTimeTarget;
use Illuminate\Database\Eloquent\Builder;

use function sprintf;

trait BaseQuery
{
    /**
     * @return QueryBuilder<Ticket>
     */
    protected function baseQuery(): QueryBuilder
    {
        $table = TicketDueTimeTarget::getTableName();

        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters($this->timeframeColumn()))
            ->leftJoin($table, 'ticket.id', '=', sprintf('%s.ticket_id', $table));
    }

    protected function timeframeColumn(): string
    {
        return sprintf('%s.created_at', TicketDueTimeTarget::getTableName());
    }

    /**
     * @return Builder<Ticket>
     */
    public function underlyingDataQuery(Request $request): Builder
    {
        $table = TicketDueTimeTarget::getTableName();

        return $this->baseQuery()
            ->with('priority')
            ->orderBy(sprintf('%s.created_at', $table))
            ->select('ticket.*', sprintf('%s.target_met', $table), sprintf('%s.breach_time', $table));
    }
}
