<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance;

use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\Cards\TableCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketNumberColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketPriorityColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketSubjectColumn;
use App\Modules\Ticket\Models\SlaPlan;
use App\Modules\Ticket\Models\Ticket;
use App\Modules\Ticket\Models\TicketDueTimeTarget;

use function sprintf;
use function trans;
use function trans_choice;

/**
 * @phpstan-extends TableCard<Ticket>
 */
class DueTimeBreachesTable extends TableCard
{
    use HasColumns;

    public function name(): string
    {
        return trans('report.due_time_breaches');
    }

    /**
     * @return QueryBuilder<Ticket>
     */
    public function query(): QueryBuilder
    {
        $targetTable = TicketDueTimeTarget::getTableName();
        $slaTable = SlaPlan::getTableName();

        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters(sprintf('%s.created_at', $targetTable)))
            ->leftJoin($targetTable, 'ticket.id', '=', sprintf('%s.ticket_id', $targetTable))
            ->leftJoin($slaTable, sprintf('%s.sla_plan_id', $targetTable), '=', sprintf('%s.id', $slaTable))
            ->with('priority')
            ->where(sprintf('%s.target_met', $targetTable), 0)
            ->orderBy(sprintf('%s.breach_time', $targetTable), 'desc')
            ->select('ticket.*', sprintf('%s.target', $targetTable), sprintf('%s.breach_time', $targetTable), sprintf('%s.name', $slaTable));
    }

    public function columns(): array
    {
        return [
            new TicketNumberColumn,
            new TicketSubjectColumn,
            $this->slaNameColumn(),
            new TicketPriorityColumn,
            $this->targetColumn(),
            $this->breachTimeColumn(),
        ];
    }

    protected function targetColumn(): Column
    {
        return (new Column(trans_choice('ticket.sla_targets', 1)))
            ->setId('target')
            ->setDefinition(['width' => '180px'])
            ->setValue(function ($model) {
                return match ($model->target) {
                    TicketDueTimeTarget::TARGET_RESOLVE_TIME     => trans('ticket.resolution_time'),
                    TicketDueTimeTarget::TARGET_FIRST_REPLY_TIME => trans('ticket.first_reply_time'),
                    TicketDueTimeTarget::TARGET_NEXT_REPLY_TIME  => trans('ticket.next_reply_time'),
                    default => '',
                };
            });
    }
}
