<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance;

use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketNumberColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketPriorityColumn;
use App\Modules\Report\Addon\Dashboards\UnderlyingData\Columns\Ticket\TicketSubjectColumn;

use function formatDate;
use function trans;
use function trans_choice;

trait HasColumns
{
    /**
     * @return array<int, Column>
     */
    public function underlyingDataTableColumns(): array
    {
        return [
            new TicketNumberColumn,
            new TicketSubjectColumn,
            $this->slaNameColumn(),
            new TicketPriorityColumn,
            $this->targetMetColumn(),
            $this->breachTimeColumn()
        ];
    }

    /**
     * @return mixed[]
     */
    public function underlyingDataTableOptions(): array
    {
        return ['ordering' => false];
    }

    protected function slaNameColumn(): Column
    {
        return (new Column(trans_choice('ticket.sla_plan', 1)))
            ->setId('slaplan')
            ->setDefinition(['width' => '180px'])
            ->setValue(fn($model) => $model->name ?? trans('general.none'));
    }

    protected function targetMetColumn(): Column
    {
        return (new Column(trans('report.target_met')))
            ->setId('target_met')
            ->setDefinition(['width' => '50px'])
            ->setValue(fn($model) => $model->target_met ? trans('general.yes') : trans('general.no'));
    }

    protected function breachTimeColumn(): Column
    {
        return (new Column(trans('report.breach_time')))
            ->setId('breach_time')
            ->setDefinition(['width' => '160px'])
            ->setValue(fn($model) => ! $model->target_met ? formatDate($model->breach_time) : '');
    }
}
