<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\SlaPerformance;

use App\Http\Requests\Request;
use App\Modules\Report\Addon\Concerns\InteractsWithTimeframe;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\ObjectDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Cards\UnderlyingData;
use App\Modules\Ticket\Models\Ticket;
use App\Modules\Ticket\Models\TicketDueTimeTarget;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Collection;

use function sprintf;

/**
 * @implements UnderlyingData<Ticket>
 */
abstract class TrendChart extends ChartCard implements UnderlyingData
{
    use BaseQuery, HasColumns, InteractsWithTimeframe;

    protected int $width = 8;
    protected int $height = 5;

    public function data(): DataStructure
    {
        $unit = $this->calculateUnits();
        $dataset = new ObjectDataset($this->getChartJsDataset()->values()->map(fn($v) => (array) $v)->all());

        return (new DataStructure(ChartType::LINE, $dataset))
            ->setAxisNames($this->unitToTrans($unit), '%')
            ->setOptions([
                'plugins' => [
                    'legend' => ['display' => false]
                ],
                'scales' => [
                    'x' => [
                        'type'    => 'time',
                        'time'    => [
                            'unit' => $unit,
                            'round' => $unit,
                            'tooltipFormat' => $this->unitToTooltipFormat($unit),
                        ],
                    ],
                    'y' => [
                        'max'   => 100,
                        'min'   => 0,
                        'ticks' => ['precision' => 0],
                    ],
                ]
            ]);
    }

    /**
     * @return Builder<Ticket>
     */
    public function underlyingDataQuery(Request $request): Builder
    {
        $table = TicketDueTimeTarget::getTableName();

        $query = $this->baseQuery()
            ->with('priority')
            ->orderBy(sprintf('%s.created_at', $table))
            ->select('ticket.*', sprintf('%s.target_met', $table), sprintf('%s.breach_time', $table));

        if ($this->isDrillIntoChartRequest($request)) {
            $time = $request->integer('value.x') / 1000;

            $query->whereRaw(sprintf('%s = ?', $this->getTimeSeriesExpr($this->timeframeColumn())), $time);
        }

        return $query;
    }

    /**
     * @return Collection<int,mixed>
     */
    private function getChartJsDataset(): Collection
    {
        return $this->baseQuery()
            ->selectRaw(sprintf('%s * 1000 as x', $this->getTimeSeriesExpr($this->timeframeColumn())))
            ->selectRaw(sprintf('count(CASE when %s.target_met = 1 THEN 1 END) / count(*) * 100 as y', TicketDueTimeTarget::getTableName()))
            ->groupBy('x')
            ->getQuery()
            ->get()
            ->keyBy('x');
    }

    private function isDrillIntoChartRequest(Request $request): bool
    {
        return $request->has('label') && $request->has('value.x') && $request->has('value.y');
    }
}
