<?php declare(strict_types=1);

namespace Addons\Reports\TicketPerformance\Reports\Cards\TimeToResolution;

use App\Modules\Report\Addon\Concerns\UnixTimestamp;
use App\Modules\Report\Addon\Dashboards\Cards\HtmlCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Report\Metrics\Metric;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Contracts\View\View;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use TemplateView;

use function trans;

class AverageCard extends HtmlCard
{
    use UnixTimestamp;

    public function name(): string
    {
        return trans('report.resolution_average');
    }

    public function html(): View
    {
        $time = (int) $this->baseQuery()->avg(DB::raw('report_metric_value.value'));
        if ($time === 0) {
            $time = trans('report.no_data_found');
        } else {
            $time = $this->formatTime($time, false);
        }

        return TemplateView::make('Reports#TicketPerformance::time')
            ->with('time', $time);
    }

    /**
     * @return QueryBuilder<Ticket>
     */
    private function baseQuery(): QueryBuilder
    {
        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket.resolved_time'))
            ->join('report_metric_value', 'report_metric_value.model_id', '=', 'ticket.id')
            ->join('report_metric', 'report_metric_value.metric_id', '=', 'report_metric.id')
            ->where('ticket.status_id', Config::get('settings.default_resolved_status'))
            ->where('report_metric.name', '=', Metric::TIME_TO_RESOLUTION->value)
            ->selectRaw('CAST(report_metric_value.value as unsigned) as resolved_time');
    }
}
