<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\TicketsOpened;

use App\Http\Requests\Request;
use App\Modules\Report\Addon\Dashboards\Cards\Table\Column;
use App\Modules\Report\Addon\Dashboards\Cards\TableCard;
use App\Modules\Report\Addon\Dashboards\Cards\UnderlyingData;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Support\Facades\DB;

use function sprintf;
use function trans;

/**
 * @phpstan-extends TableCard<Ticket>
 * @implements UnderlyingData<Ticket>
 */
abstract class BaseTable extends TableCard implements UnderlyingData
{
    protected int $width = 6;

    abstract public function queryColumn(): string;

    abstract public function modelTable(): string;

    public function modelNameAttribute(): string
    {
        return 'name';
    }

    /**
     * @inheritDoc
     */
    public function query(): QueryBuilder
    {
        return $this->baseQuery()
            ->groupBy($this->queryColumn())
            ->select([$this->queryColumn(), $this->modelNameAttribute(), DB::raw('count(*) as aggregate')]);
    }

    public function columns(): array
    {
        return [
            $this->nameColumn(),
            $this->aggregateColumn()
        ];
    }

    /**
     * @inheritDoc
     */
    public function underlyingDataQuery(Request $request): Builder
    {
        $query = $this->baseQuery();

        if ($this->isDrillIntoChartRequest($request)) {
            $query->where($this->queryColumn(), $request->integer('value'));
        }

        return $query->with($this->underlyingDataRelations())
            ->selectRaw('ticket.*');
    }

    protected function nameColumn(): Column
    {
        return (new Column(trans('general.name')))
            ->setId('name')
            ->setValue(fn($model) => $model->{$this->modelNameAttribute()});
    }

    protected function aggregateColumn(): Column
    {
        return (new Column(trans('general.count')))
            ->setId('aggregate')
            ->setValue(fn($model) => $model->aggregate);
    }

    /**
     * @return string[]
     */
    protected function underlyingDataRelations(): array
    {
        return ['user', 'department', 'status'];
    }

    /**
     * @return QueryBuilder<Ticket>
     */
    protected function baseQuery(): QueryBuilder
    {
        return QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket.created_at'))
            ->join($this->modelTable(), $this->queryColumn(), '=', sprintf('%s.id', $this->modelTable()));
    }

    protected function isDrillIntoChartRequest(Request $request): bool
    {
        return $request->has('value');
    }
}
