<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\Workload;

use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\PrimitiveDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Message;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Support\Facades\DB;

use function array_values;
use function timeWithOffset;
use function trans;
use function trans_choice;

class BusiestDayChart extends ChartCard
{
    protected int $width = 6;

    public function name(): string
    {
        return trans('report.busiest_day_of_week');
    }

    public function data(): DataStructure
    {
        return (new DataStructure(ChartType::BAR, new PrimitiveDataset($this->getChartJsDataset())))
            ->setAxisNames(trans_choice('general.day', 1), trans('report.user_messages'))
            ->setLabels([
                trans('general.sunday'),
                trans('general.monday'),
                trans('general.tuesday'),
                trans('general.wednesday'),
                trans('general.thursday'),
                trans('general.friday'),
                trans('general.saturday'),
            ])
            ->setOptions([
                'plugins' => [
                    'legend' => ['display' => false]
                ],
                'scales' => [
                    'y' => [
                        'ticks' => ['precision' => 0],
                    ],
                ],
            ]);
    }

    /**
     * @return mixed[]
     */
    private function getChartJsDataset(): array
    {
        $results = QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket_message.created_at'))
            ->leftJoin('ticket_message', 'ticket.id', '=', 'ticket_message.ticket_id')
            ->where('ticket_message.by', Message::USER)
            ->groupBy('day')
            ->orderBy('day')
            ->select([
                DB::raw('count(DISTINCT ticket_message.id) as aggregate'),
                // Need to apply timezone offset here to as we're using from_unixtime function
                DB::raw('DAYOFWEEK(from_unixtime(`ticket_message`.`created_at` - ' . timeWithOffset(0) . ')) as day')
            ])
            ->pluck('aggregate', 'day');

        $data = [];
        for ($i = 1; $i <= 7; $i++) {
            $data[$i] = $results[$i] ?? 0;
        }

        return array_values($data);
    }
}
