<?php declare(strict_types=1);

namespace Addons\Reports\TicketVolume\Reports\Cards\Workload;

use App\Modules\Report\Addon\Dashboards\Cards\Chart\ChartType;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\Datasets\CustomObjectDataset;
use App\Modules\Report\Addon\Dashboards\Cards\Chart\DataStructure;
use App\Modules\Report\Addon\Dashboards\Cards\ChartCard;
use App\Modules\Report\Addon\Dashboards\Filtering\QueryBuilder;
use App\Modules\Ticket\Models\Department;
use App\Modules\Ticket\Models\Message;
use App\Modules\Ticket\Models\Ticket;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;

use function trans;
use function trans_choice;

class DepartmentWorkloadChart extends ChartCard
{
    protected int $width = 6;

    public function name(): string
    {
        return trans('report.department_workload');
    }

    public function data(): DataStructure
    {
        $data = $this->getChartJsDataset();
        $dataset = new CustomObjectDataset($data->toArray(), 'average', 'name');

        return (new DataStructure(ChartType::BAR, $dataset))
            ->setLabels($data->pluck('name')->toArray())
            ->setAxisNames(
                trans('report.average_replies_per_assigned'),
                trans_choice('ticket.department', 1)
            )
            ->setOptions([
                'indexAxis' => 'y',
                'plugins' => [
                    'legend' => ['display' => false]
                ],
            ]);
    }

    /** @return Collection<int, array{id: int, name: string, average: (float|int)}> */
    private function getChartJsDataset(): Collection
    {
        $results = QueryBuilder::for(Ticket::class)
            ->applyFilters($this->dashboard()->filters('ticket_message.created_at'))
            ->leftJoin('ticket_message', 'ticket.id', '=', 'ticket_message.ticket_id')
            ->select(['ticket.department_id', DB::raw('count(DISTINCT ticket_message.id) as aggregate')])
            ->where('ticket_message.by', Message::OPERATOR)
            ->where('ticket_message.is_draft', 0)
            ->groupBy('ticket.department_id')
            ->pluck('aggregate', 'ticket.department_id');

        /** @var Collection<int,Department> $models */
        $models = Department::with('groups', 'operators')->get();

        return $models->map(function (Department $department) use ($results) {
            $assignedOperators = $department->fetchAssignedOperators()->count();

            return [
                'id'      => $department->id,
                'name'    => $department->name,
                'average' => isset($results[$department->id]) ? $results[$department->id] / $assignedOperators : 0,
            ];
        });
    }
}
