<?php declare(strict_types=1);

namespace SupportPal\TopoClient;

use Illuminate\Support\Collection;
use SupportPal\TopoClient\Exceptions\FileNotFoundException;
use SupportPal\TopoClient\Geometry\Type;

use function array_map;
use function collect;
use function file_get_contents;
use function json_decode;
use function sprintf;

/**
 * PHP implementation of https://github.com/topojson/topojson-client
 *
 * @phpstan-import-type Topology from Geometry
 */
class TopoClient
{
    /**
     * @return Collection<int, Feature>
     */
    public static function countries(): Collection
    {
        $path = __DIR__.'/countries-50m.json';
        $json = file_get_contents($path);
        if ($json === false) {
            throw new FileNotFoundException(sprintf('Unable to read %s.', $path));
        }

        $topology = json_decode($json, true);

        return (new TopoClient)->feature($topology, $topology['objects']['countries']);
    }

    /**
     * @param Topology $topology
     * @param mixed[] $o
     * @return Collection<int, Feature>
     */
    public function feature(array $topology, array $o): Collection
    {
        if ($o['type'] === Type::GeometryCollection->value) {
            return collect(array_map(fn ($o) => Feature::parse($topology, $o), $o['geometries']));
        }

        return collect([Feature::parse($topology, $o)]);
    }
}
