<?php declare(strict_types=1);

namespace Addons\Widgets\Announcements\Controllers;

use App\Modules\Core\Controllers\Plugins\Plugin;
use App\Modules\Core\Controllers\Plugins\Widget;
use Illuminate\Contracts\View\View as ViewContract;
use SimplePie\SimplePie;
use SupportPal\ProxyService\ProxyService;
use View;

use function app;
use function cache;

class Announcements extends Plugin implements Widget
{
    public function getEmbeddableView(): ?ViewContract
    {
        // Process the RSS Load
        $rss = app()->make(SimplePie::class);
        $rss->set_curl_options(app(ProxyService::class)->curlOpts());
        $rss->set_feed_url('https://www.supportpal.com/support/announcements/rss');
        $rss->set_cache(cache()->driver());
        $rss->init();

        $announcements = [];
        foreach ((array) $rss->get_items(0, 2) as $announcement) {
            $announcements[] = new Announcement($announcement);
        }

        return View::make('Widgets#Announcements::view', ['announcements' => $announcements]);
    }

    /**
     * Plugins can run an installation routine when they are activated. This
     * will typically include adding default values, initialising database tables
     * and so on.
     *
     * @return mixed
     */
    public function activate()
    {
        return true;
    }

    /**
     * Deactivating serves as temporarily disabling the plugin, but the files still
     * remain. This function should typically clear any caches and temporary directories.
     *
     * @return mixed
     */
    public function deactivate()
    {
        return true;
    }

    /**
     * When a plugin is uninstalled, it should be completely removed as if it never
     * was there. This function should delete any created database tables, and any files
     * created outside of the plugin directory.
     *
     * @return mixed
     */
    public function uninstall()
    {
        return true;
    }
}
