<?php declare(strict_types=1);

namespace Addons\Widgets\GettingStarted\Controllers;

use ArrayAccess;

/**
 * @implements ArrayAccess<mixed, mixed>
 */
abstract class Step implements ArrayAccess
{
    /**
     * Configuration options, indexed by configuration key
     *
     * @var array<mixed, mixed>
     */
    private $data = [];

    /**
     * Initialise a getting started step
     *
     * @param  string $name Name of the step
     * @param  string $url  URL to begin the step
     */
    public function __construct($name = '', $url = '')
    {
        $this->data = [
            'name' => $name,
            'url'  => $url,
        ];
    }

    /**
     * Determine whether a step has been completed.
     *
     * @return boolean
     */
    abstract public function isComplete();

    /**
     * Get a data by key
     */
    public function __get(string $key): mixed
    {
        if ($key === 'complete') {
            return $this->isComplete();
        }

        return isset($this->data[$key]) ? $this->data[$key] : '';
    }

    /**
     * Assigns a value to the specified data
     */
    public function __set(string $key, mixed $value): void
    {
        $this->data[$key] = $value;
    }

    /**
     * Whether or not an data exists by key
     */
    public function __isset(string $key): bool
    {
        return isset($this->data[$key]);
    }

    /**
     * Unsets an data by key
     */
    public function __unset(string $key): void
    {
        unset($this->data[$key]);
    }

    /**
     * Assigns a value to the specified offset
     */
    public function offsetSet(mixed $offset, mixed $value): void
    {
        if ($offset === null) {
            $this->data[] = $value;
        } else {
            $this->data[$offset] = $value;
        }
    }

    /**
     * Whether or not an offset exists
     */
    public function offsetExists(mixed $offset): bool
    {
        return isset($this->data[$offset]);
    }

    /**
     * Unsets an offset
     */
    public function offsetUnset(mixed $offset): void
    {
        if (! $this->offsetExists($offset)) {
            return;
        }

        unset($this->data[$offset]);
    }

    /**
     * Returns the value at specified offset
     */
    public function offsetGet(mixed $offset): mixed
    {
        if ($offset === 'complete') {
            return $this->isComplete();
        }

        return $this->offsetExists($offset) ? $this->data[$offset] : '';
    }
}
