<?php declare(strict_types=1);

namespace SupportPal\Support\Utils;

use Illuminate\Contracts\Cache\Repository;

use function bin2hex;
use function function_exists;
use function opcache_reset;
use function random_bytes;

class OpcacheUtil
{
    private const CACHE_KEY = 'opcache_clear_cache_secret';

    private Repository $cacheManager;

    public function __construct(Repository $cacheManager)
    {
        $this->cacheManager = $cacheManager;
    }

    public function createTemporarySecret(): string
    {
        $key = bin2hex(random_bytes(16));
        $this->cacheManager->put(self::CACHE_KEY, $key, 60);

        return $key;
    }

    public function isTemporarySecret(string $secret): bool
    {
        return $this->cacheManager->get(self::CACHE_KEY) === $secret;
    }

    public function forgetTemporarySecret(): bool
    {
        return $this->cacheManager->forget(self::CACHE_KEY);
    }

    public function reset(): void
    {
        if (! function_exists('opcache_reset')) {
            return;
        }

        opcache_reset();
    }
}
