<?php

use Illuminate\Support\Facades\Facade;

return [

    /*
    |--------------------------------------------------------------------------
    | Application Name
    |--------------------------------------------------------------------------
    |
    | This value is the name of your application. This value is used when the
    | framework needs to place the application's name in a notification or
    | any other location as required by the application or its packages.
    |
    */

    'name' => env('APP_NAME', 'SupportPal'),

    /*
    |--------------------------------------------------------------------------
    | Application Environment
    |--------------------------------------------------------------------------
    |
    | The application environment enables you to override base configuration
    | items in ./config/*.php. This is useful for development purposes. The
    | environment defaults to production. This can be overridden using $_ENV.
    |
    */

    'env' => env('APP_ENV', 'production'),

    /*
    |--------------------------------------------------------------------------
    | Application Debug Mode
    |--------------------------------------------------------------------------
    |
    | When your application is in debug mode, detailed error messages with
    | stack traces will be shown on every error that occurs within your
    | application. If disabled, a simple generic error page is shown.
    |
    */

    'debug' => env('APP_DEBUG', false),

    /*
    |--------------------------------------------------------------------------
    | Application URL
    |--------------------------------------------------------------------------
    |
    | This URL is used by the console to properly generate URLs when using
    | the Artisan command line tool. You should set this to the root of
    | your application so that it is used when running Artisan tasks.
    |
    */

    'url' => 'http://localhost',

    'asset_url' => env('ASSET_URL'),

    'api_prefix' => env('API_PREFIX', 'api'),

    /*
    |--------------------------------------------------------------------------
    | Application Timezone
    |--------------------------------------------------------------------------
    |
    | Here you may specify the default timezone for your application, which
    | will be used by the PHP date and date-time functions. We have gone
    | ahead and set this to a sensible default for you out of the box.
    |
    */

    'timezone' => 'UTC',

    /*
    |--------------------------------------------------------------------------
    | Application Locale Configuration
    |--------------------------------------------------------------------------
    |
    | The application locale determines the default locale that will be used
    | by the translation service provider. You are free to set this value
    | to any of the locales which will be supported by the application.
    |
    */

    'locale' => 'en',

    /*
    |--------------------------------------------------------------------------
    | Application Fallback Locale
    |--------------------------------------------------------------------------
    |
    | The fallback locale determines the locale to use when the current one
    | is not available. You may change the value to correspond to any of
    | the language folders that are provided through your application.
    |
    */

    'fallback_locale' => 'en',

    /*
    |--------------------------------------------------------------------------
    | Faker Locale
    |--------------------------------------------------------------------------
    |
    | This locale will be used by the Faker PHP library when generating fake
    | data for your database seeds. For example, this will be used to get
    | localized telephone numbers, street address information and more.
    |
    */
    'faker_locale' => env('FAKER_LOCALE', 'en_US'),

    /*
    |--------------------------------------------------------------------------
    | Encryption Key
    |--------------------------------------------------------------------------
    |
    | This key is used by the Illuminate encrypter service and should be set
    | to a random, 32 character string, otherwise these encrypted strings
    | will not be safe. Please do this before deploying an application!
    |
    */

    'key' => env('APP_KEY', 'SomeRandomStringSomeRandomString'),

    'cipher' => 'AES-256-CBC',

    /*
    |--------------------------------------------------------------------------
    | Log Path
    |--------------------------------------------------------------------------
    |
    | Absolute path to directory where application logs are stored. The
    | directory must be writable by both your web server and CLI.
    |
    */

    'log_path' => storage_path().'/logs',

    /*
    |--------------------------------------------------------------------------
    | Autoloaded Service Providers
    |--------------------------------------------------------------------------
    |
    | The service providers listed here will be automatically loaded on the
    | request to your application. Feel free to add your own services to
    | this array to grant expanded functionality to your applications.
    |
    */

    'providers' => [

        /*
         * Laravel Framework Service Providers...
         */
        Illuminate\Auth\AuthServiceProvider::class,
        Illuminate\Broadcasting\BroadcastServiceProvider::class,
        Illuminate\Bus\BusServiceProvider::class,
        Illuminate\Cache\CacheServiceProvider::class,
        //Illuminate\Foundation\Providers\ConsoleSupportServiceProvider::class,
        Illuminate\Database\MigrationServiceProvider::class,
        Illuminate\Foundation\Providers\ComposerServiceProvider::class,
        Illuminate\Cookie\CookieServiceProvider::class,
        App\Providers\DatabaseServiceProvider::class,
        Illuminate\Encryption\EncryptionServiceProvider::class,
        App\Providers\FilesystemServiceProvider::class,
        Illuminate\Foundation\Providers\FoundationServiceProvider::class,
        Illuminate\Hashing\HashServiceProvider::class,
        Illuminate\Notifications\NotificationServiceProvider::class,
        Illuminate\Pagination\PaginationServiceProvider::class,
        Illuminate\Pipeline\PipelineServiceProvider::class,
        Illuminate\Queue\QueueServiceProvider::class,
        Illuminate\Redis\RedisServiceProvider::class,
        Illuminate\Auth\Passwords\PasswordResetServiceProvider::class,
        Laravel\Scout\ScoutServiceProvider::class,
        App\Providers\SessionServiceProvider::class,
        App\Providers\TranslationServiceProvider::class,
        Illuminate\Validation\ValidationServiceProvider::class,

        /*
         * Load these Package Service Providers before ours.
         */
        SupportPal\Pollcast\ServiceProvider::class,
        SupportPal\Support\ServiceProvider::class,
        TwigBridge\ServiceProvider::class,
        SupportPal\ProxyService\ProxyServiceProvider::class,
        SupportPal\OAuth\OAuthServiceProvider::class,
        Neves\Events\EventServiceProvider::class,

        /*
         * Application Service Providers...
         */
        App\Providers\ViewServiceProvider::class,
        App\Providers\DeferredServiceProvider::class,
        App\Providers\SettingServiceProvider::class,
        SupportPal\Marketplace\ClientServiceProvider::class,
        App\Providers\AddonServiceProvider::class,
        App\Providers\AppServiceProvider::class,
        App\Providers\EmailServiceProvider::class,
        App\Providers\AuthServiceProvider::class,
        App\Providers\EventServiceProvider::class,
        App\Providers\ModuleServiceProvider::class,
        App\Providers\BroadcastServiceProvider::class,
        App\Providers\PurifierServiceProvider::class,
        App\Providers\SamlServiceProvider::class,
        App\Providers\OAuthServiceProvider::class,
        App\Providers\ArtisanServiceProvider::class,
        App\Providers\HtmlServiceProvider::class,
        App\Providers\HorizonServiceProvider::class,
        App\Providers\HealthCheckProvider::class,
        App\Providers\PhoneVerifyServiceProvider::class,
        App\Providers\LicensingChannelServiceProvider::class,
        App\Providers\SearchServiceProvider::class,
        App\Providers\CachableCallsServiceProvider::class,
        App\Providers\ReportServiceProvider::class,

        /*
         * Package Service Providers...
         */
        Chumper\Datatable\DatatableServiceProvider::class,
        Kodeine\Acl\AclServiceProvider::class,
        Proengsoft\JsValidation\JsValidationServiceProvider::class,
        JsLocalization\JsLocalizationServiceProvider::class,
        Lord\Laroute\LarouteServiceProvider::class,
        Intervention\Image\ImageServiceProvider::class,
        Propaganistas\LaravelPhone\PhoneServiceProvider::class,
        BeyondCode\LaravelWebSockets\WebSocketsServiceProvider::class,

    ],

    /*
    |--------------------------------------------------------------------------
    | Class Aliases
    |--------------------------------------------------------------------------
    |
    | This array of class aliases will be registered when this application
    | is started. However, feel free to register as many as you wish as
    | the aliases are "lazy" loaded so they don't hinder performance.
    |
    */

    'aliases' => Facade::defaultAliases()->merge([
        'Captcha'           => App\Modules\Core\Controllers\Captchas\Facades\CaptchaFactory::class,
        'Carbon'            => Carbon\Carbon::class,
        'ChannelFactory'    => App\Modules\Ticket\Controllers\Channels\Facades\ChannelFactory::class,
        'Datatable'         => Chumper\Datatable\Facades\DatatableFacade::class,
        'Form'              => Collective\Html\FormFacade::class,
        'Html'              => Collective\Html\HtmlFacade::class,
        'Image'             => Intervention\Image\Facades\Image::class,
        'JsValidator'       => Proengsoft\JsValidation\Facades\JsValidatorFacade::class,
        'LanguageFactory'   => App\Modules\Core\Controllers\Languages\Facades\LanguageFactory::class,
        'Mailer'            => App\Modules\Core\Controllers\Mailer\Mailer::class,
        'PluginFactory'     => App\Modules\Core\Controllers\Plugins\Facades\PluginFactory::class,
        'Purifier'          => SupportPal\Purifier\Facades\Purifier::class,
        'SamlAuth'          => App\Modules\User\Controllers\Saml\Facades\SamlAuthManager::class,
        'TemplateView'      => App\Modules\Core\Controllers\Templates\Facade::class,
        'Twig'              => TwigBridge\Facade\Twig::class,
        'WidgetFactory'     => App\Modules\Core\Controllers\Plugins\Facades\WidgetFactory::class,
    ])->toArray(),

    /*
    |--------------------------------------------------------------------------
    | Ticket throttling
    |--------------------------------------------------------------------------
    |
    | The maximum number of tickets a user is allowed to create within the
    | decay period before further attempts are blocked.
    |
    | For example, by default a user can create 3 tickets within 5 minutes.
    |
    */
    'ticket_throttle_max_requests' => env('TICKET_THROTTLE_MAX_REQUESTS', 3),
    'ticket_throttle_decay'        => env('TICKET_THROTTLE_DECAY', 5),

    /*
    |--------------------------------------------------------------------------
    | Email throttling
    |--------------------------------------------------------------------------
    |
    | The maximum number of emails a user is allowed to send to the helpdesk
    | within the decay period before further attempts are blocked.
    |
    | For example, by default a the same email address can send 5 emails to
    | the help desk within 15 minutes.
    |
    */
    'email_throttle_max_requests'  => env('EMAIL_THROTTLE_MAX_REQUESTS', 5),
    'email_throttle_decay'         => env('EMAIL_THROTTLE_DECAY', 15),

    /*
    |--------------------------------------------------------------------------
    | LDAP Timeout
    |--------------------------------------------------------------------------
    |
    | The number of seconds to wait before timing out the connection
    | to the LDAP server.
    |
    */
    'ldap_timeout'                 => env('LDAP_TIMEOUT', 5),

    /*
    |--------------------------------------------------------------------------
    | Ticket Thrash
    |--------------------------------------------------------------------------
    |
    | The number of days after which tickets in the trash are deleted
    | permanently automatically, counts from when the ticket was moved
    | to the trash.
    |
    */
    'empty_ticket_trash'           => env('EMPTY_TICKET_TRASH', 30),

    /*
    |--------------------------------------------------------------------------
    | Ticket Message Performance
    |--------------------------------------------------------------------------
    |
    | The number of messages to pre load when viewing a ticket irrespective
    | of max_message_size.
    |
    | Set to null to pre load all messages. Note: this may introduce
    | performance issues when loading large tickets with large messages.
    |
    | Messages which are not preloaded will show an excerpt and the full text
    | will be loaded dynamically when viewed by an end user.
    |
    */
    'message_load_no'              => env('MESSAGE_LOAD_NO', 2),

    /*
    |--------------------------------------------------------------------------
    | Ticket Message Performance
    |--------------------------------------------------------------------------
    |
    | This setting relates to message_load_no and is best described through
    | an example. If a ticket contains 40 messages and message_load_no is 2,
    | there are 38 remaining messages. When message_load_limit is 20, 20 of
    | those 38 messages, which are less than max_message_size, will be
    | preloaded.
    |
    | Set to null to pre load all messages which are less than max_message_size.
    | Note: this may introduce performance issues when loading large tickets
    | with large messages.
    |
    | Messages which are not pre loaded will show an excerpt and the full text
    | will be loaded dynamically when viewed by an end user.
    |
    */
    'message_load_limit'           => env('MESSAGE_LOAD_LIMIT', 20),

    /*
    |--------------------------------------------------------------------------
    | Ticket Message Performance
    |--------------------------------------------------------------------------
    |
    | If we wish to automatically load the latest (default) or earliest
    | message_load_limit messages when loading the ticket.
    |
    | Valid values: asc, desc
    |
    */
    'message_load_order'           => env('MESSAGE_LOAD_ORDER', 'desc'),

    /*
    |--------------------------------------------------------------------------
    | Ticket Message Performance
    |--------------------------------------------------------------------------
    |
    | Messages that are greater than this size in bytes will be truncated.
    | The original message can be viewed by clicking the "View entire message"
    | link at the bottom of the message.
    |
    | It is not possible to disable this setting. Set it to a high value
    | instead. Beware, browsers will typically crash if messages are too big.
    |
    */
    'max_message_size'             => env('MESSAGE_MAX_SIZE', 102000),

    /*
    |--------------------------------------------------------------------------
    | Email Log Expiry
    |--------------------------------------------------------------------------
    |
    | The number of days that raw email log files are stored on the file
    | system. Emails from pre-2.4.0 are stored in the database and must be
    | pruned via the System Cleanup tool.
    |
    */
    'raw_email_expiry_days'        => env('RAW_EMAIL_EXPIRY_DAYS', 30),

    /*
    |--------------------------------------------------------------------------
    | 2FA Account Name
    |--------------------------------------------------------------------------
    |
    | The default account name that shows when setting up 2FA.
    |
    */
    '2fa_name'                     => env('2FA_NAME'),

    /*
    |--------------------------------------------------------------------------
    | Disable SSL
    |--------------------------------------------------------------------------
    |
    | Disable SSL redirection middleware.
    | Use only when Enable SSL is toggled on in SupportPal and HTTPS
    | connections are broken.
    |
    */
    'force_disable_ssl'            => env('FORCE_DISABLE_SSL', false),

    /*
    |--------------------------------------------------------------------------
    | LDAP Dual Login
    |--------------------------------------------------------------------------
    |
    | If operators can log in with their regular operator login as well as
    | their LDAP account.
    |
    */
    'ldap_dual_login'              => env('LDAP_DUAL_LOGIN', false),

    /*
    |--------------------------------------------------------------------------
    | Self-service Articles
    |--------------------------------------------------------------------------
    |
    | The number of articles to show per category in the self-service by
    | default, for 'hierarchical' types only.
    |
    */
    'selfservice_articles'         => env('TOTAL_SELFSERVICE_ARTICLES', 3),

    /*
    |--------------------------------------------------------------------------
    | Total Visible Recipients
    |--------------------------------------------------------------------------
    |
    | The total number of recipients that should be visible when viewing the
    | e-mail log. If there are more addresses, "and x others" will be shown.
    |
    */
    'total_visible_recipients'     => env('TOTAL_VISIBLE_RECIPIENTS', 2),

    /*
    |--------------------------------------------------------------------------
    | Timeago
    |--------------------------------------------------------------------------
    |
    | Toggle to disable relative times (e.g. '1 hour ago') and use absolute
    | dates instead.
    |
    */
    'timeago'                      => env('ENABLE_TIMEAGO', true),

];
