<?php

return [

    /*
    |--------------------------------------------------------------------------
    | Mail Driver
    |--------------------------------------------------------------------------
    |
    | Laravel supports both SMTP and PHP's "mail" function as drivers for the
    | sending of e-mail. You may specify which one you're using throughout
    | your application here. By default, Laravel is setup for SMTP mail.
    |
    | Supported: "global"
    |
    */

    'default' => 'global',

    /*
    |--------------------------------------------------------------------------
    | SMTP Local Domain
    |--------------------------------------------------------------------------
    |
    | Set your servers hostname, which will be used in the EHLO statement
    | to the SMTP server. Defaults to $_SERVER['SERVER_NAME'] or "[127.0.0.1]"
    | when the previous value is unavailable (such as CLI).
    |
    */

    'local_domain' => env('MAIL_LOCAL_DOMAIN'),

    /*
    |--------------------------------------------------------------------------
    | SMTP Socket Address
    |--------------------------------------------------------------------------
    |
    | Used to specify the IP address (either IPv4 or IPv6) and/or the port
    | number that PHP will use to access the network. The syntax is ip:port for
    | IPv4 addresses, and [ip]:port for IPv6 addresses.
    | See https://www.php.net/manual/en/context.socket.php
    |
    */

    'source_ip' => env('MAIL_SOURCE_IP'),

    /*
    |--------------------------------------------------------------------------
    | SMTP Server Timeout
    |--------------------------------------------------------------------------
    |
    | The number of seconds that the system should wait for a response
    | before timing out.
    |
    */

    'timeout' => env('MAIL_TIMEOUT', 15),

    /*
    |--------------------------------------------------------------------------
    | Security Policy
    |--------------------------------------------------------------------------
    |
    | All outbound emails are bound to the below security policy. This governs
    | what is possible from within e-mail templates and ensure that rogue
    | staff members do not result in a security risk.
    |
    */

    'policy' => \App\Modules\Core\Controllers\Mailer\MailPolicy::policy()->toArray(),

    /*
    |--------------------------------------------------------------------------
    | Clipping Delimiter
    |--------------------------------------------------------------------------
    |
    | Character sequence wrapped around "Please respond above this line"
    | message added to the start of all ticket emails. Used to identify the end
    | of replies via email by searching for a HTML node that starts and ends
    | with the character sequence.
    |
    | Take extreme care when editing as replies via the email channel may not
    | be clipped or may be clipped incorrectly.
    |
    | Valid characters: alphanumeric, dashes, hashes, exclamation marks.
    |
    */

    'delimiter' => env('MAIL_DELIMITER', '-#-#-'),

    /*
    |--------------------------------------------------------------------------
    | Reply Line
    |--------------------------------------------------------------------------
    |
    | The "Please reply above this line" message added to the start of ticket
    | emails generated by SupportPal.
    |
    | The %s inside a HTML node is REQUIRED for clipping replies via email. It
    | is automatically replaced by "mail.delimiter Please reply above this line
    | mail.delimiter". The language variable can be found in the language file
    | resources\lang\*\ticket.php under key "reply_above_line".
    |
    | Set to an empty string to completely remove reply line. This means
    | replies via the email channel will not be clipped.
    |
    */

    'reply_line' => env('MAIL_REPLY_LINE', '<em style="color: #999;">%s</em><br /><br />'),

];
