<?php declare(strict_types=1);

namespace Database\Migrations\v5_0_0\Selfservice;

use App\Modules\Core\Models\Repository\SettingsRepository;
use DB;
use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

class UpdateArticleTranslationTable extends Migration
{
    private string $table = 'article_translation';

    /**
     * Run the migrations.
     */
    public function up(): void
    {
        if (app()->make(SettingsRepository::class)->isInstalled()) {

            DB::unprepared(<<<STATEMENT
ALTER TABLE article_translation
    ADD COLUMN id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
    DROP INDEX article_translation_title_ft_index,
    DROP INDEX article_translation_plain_text_ft_index;
STATEMENT);
        } else {
            Schema::table($this->table, function (Blueprint $table) {
                $table->dropFullText('article_translation_title_ft_index');
                $table->dropFullText('article_translation_plain_text_ft_index');
            });
        }

        Schema::table($this->table, function (Blueprint $table) {
            $table->fullText('title');
            $table->fullText('plain_text');
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::table($this->table, function (Blueprint $table) {
            $table->dropFullText(['title']);
            $table->dropFullText(['plain_text']);
        });

        Schema::table($this->table, function (Blueprint $table) {
            $table->dropPrimary();
            $table->dropColumn('id');
        });

        Schema::table($this->table, function (Blueprint $table) {
            $table->fullText('title');
            $table->fullText('plain_text');
        });
    }
}
