<?php declare(strict_types=1);

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::table('operator_private_conversation', function (Blueprint $table) {
            $table->string('subject')->nullable()->after('to_user_id');

            // Need to drop foreign index first
            $table->dropForeign([ 'from_user_id' ]);
            $table->dropForeign([ 'to_user_id' ]);

            // Remove unique index
            $table->dropUnique('operator_private_conversation_unique');

            // Add foreign index back
            $table->foreign('from_user_id')->references('id')->on('user')->onDelete('cascade');
            $table->foreign('to_user_id')->references('id')->on('user')->onDelete('cascade');
        });

        $subquery = '(SELECT `updated_at` FROM `operator_private_message` WHERE `operator_private_conversation`.`id` = `operator_private_message`.`conversation_id` ORDER BY `operator_private_message`.`updated_at` DESC LIMIT 1)';
        DB::table('operator_private_conversation')
            ->update(['updated_at' => DB::raw($subquery)]);
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::table('operator_private_conversation', function (Blueprint $table) {
            $table->dropColumn('subject');

            // Need to drop foreign index first
            $table->dropForeign([ 'from_user_id' ]);
            $table->dropForeign([ 'to_user_id' ]);

            // Add unique index
            $table->unique(['from_user_id', 'to_user_id'], 'operator_private_conversation_unique');

            // Add foreign index back
            $table->foreign('from_user_id')->references('id')->on('user')->onDelete('cascade');
            $table->foreign('to_user_id')->references('id')->on('user')->onDelete('cascade');
        });
    }
};
