<?php declare(strict_types=1);

namespace Database\Seeds\v5_0_0;

use App\Exceptions\Handler;
use App\Modules\Core\Controllers\Addons\Manager\AddonManager;
use App\Modules\Core\Controllers\Database\Seed\Seeder;
use Exception;
use Illuminate\Contracts\Container\Container;
use Illuminate\Support\Facades\DB;
use SupportPal\Marketplace\Http\Api\Addons;

use function sprintf;

class SeedChannelTable extends Seeder
{
    private Addons $addonsApi;

    private AddonManager $addonManager;

    public function __construct(Container $container, Addons $addonsApi, AddonManager $addonManager)
    {
        parent::__construct($container);

        $this->addonsApi = $addonsApi;
        $this->addonManager = $addonManager;
    }

    /**
     * Run the database seeds.
     *
     * @return void
     */
    public function run()
    {
        $coreChannels = ['Facebook', 'HelpWidget', 'Twilio', 'Twitter'];

        $channels = DB::table('ticket_channel')->whereIn('name', $coreChannels)->where('enabled', 1)->get();

        foreach ($channels as $channel) {
            $addons = $this->addonsApi->get(['categories' => ['channels' => [$channel->name]]]);

            if ($addons->isEmpty()) {
                $this->note(sprintf('Unable to find channel %s at marketplace.', $channel->name));

                continue;
            }

            try {
                $addon = $addons->first();
                $this->addonManager->update($addon);
            } catch (Exception $e) {
                if ($this->container->runningUnitTests()) {
                    throw $e;
                }

                Handler::reportException($e);
                $this->note(sprintf('Failed to update channel %s from marketplace.', $channel->name));
            }
        }
    }
}
