//
// $('#element').donetyping(callback[, timeout=1000])
// Fires callback when a user has finished typing. This is determined by the time elapsed
// since the last keystroke and timeout parameter or the blur event--whichever comes first.
//   @callback: function to be called when even triggers
//   @timeout:  (default=1000) timeout, in ms, to to wait before triggering event if not
//              caused by blur.
// Requires jQuery 1.7+
//
;
(function ($) {
  $.fn.extend({
    donetyping: function (callback, timeout) {
      timeout = timeout || 1e3; // 1 second default timeout
      var timeoutReference,
        doneTyping = function (el) {
          if (!timeoutReference) return;
          timeoutReference = null;
          if (callback) {
            callback.call(el);
          }
          $(el).trigger('donetyping');
        };
      return this.each(function (i, el) {
        var $el = $(el);
        // Check keyup event, also added paste to catch when you copy and paste something without typing
        $el.is(':input') && $el.on('paste keyup', function () {
          // Check if timeout has been set. If it has, "reset" the clock and start over again.
          if (timeoutReference) clearTimeout(timeoutReference);
          timeoutReference = setTimeout(function () {
            // if we made it here, our timeout has elapsed. Fire the callback
            doneTyping(el);
          }, timeout);
        }).on('blur', function () {
          // If we can, fire the event since we're leaving the field
          doneTyping(el);
        });
      });
    }
  });
})(jQuery);