<?php declare(strict_types=1);

namespace SupportPal\Support\Utils;

use function array_replace_recursive;
use function dirname;
use function file_exists;
use function file_put_contents;
use function is_dir;
use function mkdir;
use function rename;
use function time;
use function var_export;

use const LOCK_EX;

abstract class ConfigHelper
{
    /**
     * @param mixed[] $default
     * @param mixed[] $config
     */
    public function createConfigFile(string $name, array $default, array $config = []): void
    {
        $path = $this->getConfigPathname($name);
        if (! is_dir(dirname($path))) {
            mkdir(dirname($path), 0755, true);
        }

        // Application already has a config file, so let's back it up.
        $existing = [];
        if (file_exists($path)) {
            $existing = include $path;
            @ rename($path, $path.'.'.time().'.bak');
        }

        // Merge the configs. The config file which already exists takes precedence over the default, but not over
        // user entered config.
        $data = '<?php return '
            . var_export(array_replace_recursive($default, $existing, $config), true)
            . ';';

        file_put_contents($path, $data, LOCK_EX);
    }

    /**
     * Get the absolute path to the config directory with environment.
     */
    abstract public function getConfigPathname(string $name): string;
}
